import 'dart:ui';

import 'package:flutter/services.dart';

// class ActivationMethod {
//   static const String NONE = "NONE";
//   static const String SHAKE = "SHAKE";
//   static const String SCREENSHOT = "SCREENSHOT";
// }

enum ActivationMethod { NONE, SHAKE, SCREENSHOT }

String getActivationMethodValue(ActivationMethod activationMethod) {
  switch (activationMethod) {
    case ActivationMethod.NONE:
      return "NONE";
    case ActivationMethod.SHAKE:
      return "SHAKE";
    case ActivationMethod.SCREENSHOT:
      return "SCREENSHOT";
    default:
      return "NONE";
  }
}

enum BugPriority { LOW, MEDIUM, HIGH }

String getBugPriorityValue(BugPriority bugPriority) {
  switch (bugPriority) {
    case BugPriority.LOW:
      return "LOW";
    case BugPriority.MEDIUM:
      return "MEDIUM";
    case BugPriority.HIGH:
      return "HIGH";
    default:
      return "MEDIUM";
  }
}

class BugBattle {
  static const MethodChannel _channel = const MethodChannel('bugbattle_sdk');

  /// initializes bugbattle with [apiKey] and sets the [activationMethod]
  /// ### required
  static void initWithToken(String apiKey, ActivationMethod activationMethod) {
    String activationMethodValue = getActivationMethodValue(activationMethod);
    _channel.invokeMethod(
      'initWithToken',
      {'apiKey': apiKey, 'activationMethod': activationMethodValue},
    );
  }

  /// initializes bugbattle with [apiKey] and sets the array of [activationMethods]
  static void initializeMany(
      String apiKey, List<ActivationMethod> activationMethods) {
    List<String> activationMethodsValues = [];
    for (int i = 0; i < activationMethods.length; i++) {
      String activationMethodString =
          getActivationMethodValue(activationMethods[i]);
      activationMethodsValues.add(activationMethodString);
    }

    _channel.invokeMethod('initializeMany',
        {'apiKey': apiKey, "activationMethods": activationMethodsValues});
  }

  /// starts bug reporting
  static startBugReporting() {
    _channel.invokeMethod('startBugReporting');
  }

  /// enables replays
  static void enableReplays() {
    _channel.invokeMethod('enableReplays');
  }

  /// sets a customer email
  static void setCustomerEmail(String email) {
    _channel.invokeMethod('setCustomerEmail', {"email": email});
  }

  /// sends a silent bug report
  static void sendSilentBugReport(
    String email,
    String description,
    BugPriority priority,
  ) {
    String bugPriorityVal = getBugPriorityValue(priority);

    _channel.invokeMethod('sendSilentBugReport', {
      "email": email,
      "description": description,
      "priority": bugPriorityVal
    });
  }

  /// attachs custom data
  static void attachCustomData(Map<String, dynamic> customData) async {
    _channel.invokeMethod('attachCustomData', {"customData": customData});
  }

  /// [enable] enables the pricacy policy.
  static void enablePrivacyPolicy(bool enable) {
    _channel.invokeMethod('enablePrivacyPolicy', {"enable": enable});
  }

  /// Sets the privacy url url to [url].
  static void setPrivacyPolicyUrl(String url) {
    _channel.invokeMethod('setPrivacyPolicyUrl', {"url": url});
  }

  /// ### IOS only.
  static void setNavigationBarTint(Color color) {
    final String hexString = _colorToHex(color);
    _channel.invokeMethod('setNavigationBarTint', {"color": hexString});
  }

  /// ### IOS only.
  static void setNavigationTint(Color color) {
    final String hexString = _colorToHex(color);
    _channel.invokeMethod('setNavigationTint', {"color": hexString});
  }

  /// ### IOS only.
  static void setNavigationBarTitleColor(Color color) {
    final String hexString = _colorToHex(color);
    _channel.invokeMethod('setNavigationBarTitleColor', {"color": hexString});
  }

  /// Sets the api url to [url].
  static void setApiUrl(String url) {
    _channel.invokeMethod('setApiUrl', {"url": url});
  }

  static String _colorToHex(Color color) {
    return '#${(color.value & 0xFFFFFF).toRadixString(16).padLeft(6, '0').toUpperCase()}';
  }

  static void setLanguage(String language) {
    _channel.invokeMethod('setLanguage', {"language": language});
  }
}
